"""
Wrapper class to write beautiful
generic LaTeX files.
"""

from pygments import highlight
from pygments.formatters import LatexFormatter
from pygments.lexers.functional import OcamlLexer

OL = OcamlLexer()

class CamlTexFileWriter(object):
    """
    Wrapper class to manage listings in OCaml.
    """
    def __init__(self, filepath, style = 'default', prompt=None):
        self.formatter = LatexFormatter(style=style)
        self.prompt = prompt

        self.fname = filepath
        self.fpointer = open(filepath, 'w')

    def write_styles(self):
        """
        Make it so that the OCaml looks
        pretty, by outputting the appropriate
        code formatting styles.
        """
        self.fpointer.write('\n\\usepackage{fancyvrb,color}\n')
        self.fpointer.write(self.formatter.get_style_defs())

    def write_tex(self, line):
        """
        Return a line of LaTeX to the file.
        """
        self.fpointer.write(line)
        return True

    def write_ocaml(self, ml_block):
        self.fpointer.write(highlight(ml_block, OL, self.formatter))
        self.fpointer.flush()

    def write_ocaml_with_evals(self, ml_statements):
        
        def clean(s):

            splitted = s.split(';;\n')
            
            if len(splitted) > 1:
                stmnt = splitted[0] + ';;'
                evaled = splitted[1].strip()
                return stmnt + '\n' + evaled
            else:
                return s

        statements = [clean(mls) for mls in ml_statements]
        self.write_ocaml_statements(statements)


    def write_ocaml_statements(self, statements):
        """
        Write stylized OCaml to the output file,
        styling with pygments at the same time.
        """
        if self.prompt:
            statements = ["{} {}".format(self.prompt, s) for s in statements]
        
        self.write_ocaml("\n".join(statements))

        return True

    def close(self):
        """Close the file writer"""
        self.fpointer.close()

    def __repr__(self):
        return "<CamlWriter {}>".format(self.fname)



"""
Module containing wrapper class for an
OCamlSession, built on pexpect.
"""


import pexpect
import re
import sys

ANSI_RE = re.compile(r'\x1b[^m]*m')
ANSI_RE2 = re.compile(r'\x1b\[A')

class OCamlSession(object):
    """
    Wrapper around the pexpect class to run an OCaml session.

    Might make it easier to this to be a generalized tool!
    """

    def __init__(self):
        self.ocaml = pexpect.spawn('ocaml')
        self.ocaml.expect('#')

    def evaluate(self, ml_block):
        """
        Given an ML block, return the result 
        of evaluating the ML block
        in the toplevel.
        """
        if '#' in ml_block:
            count = len([c for c in ml_block if c == '#'])
        else:
            count = 0

        # self.ocaml.sendline(unicode(ml_block))
        self.ocaml.sendline(ml_block)

        statement = ""
        self.ocaml.expect(r'#\s')
        statement +=  ANSI_RE2.sub('', ANSI_RE.sub('', self.ocaml.before))

        statement = statement.strip()
        statement = statement.replace('\r\n', '\n')
        statement = statement.replace('\n\n', '\n')

        return statement.strip()

    def reset(self):
        """
        Return the session to an clean state by
        resetting the underlying OCaml process.
        """
        self.ocaml = pexpect.spawn('ocaml', encoding='ascii')
        # self.ocaml = pexpect.spawnu('ocaml -stdin') # , encoding='ascii')

        self.ocaml.expect('#')

        return True

    def __repr__(self):
        return "<OCaml Session>"



from setuptools import setup, find_packages

setup(name='camltex',
        version='0.1.2',
        description='CLI for processing embedded OCaml.',
        url='http://github.com/cs51/camltex',
        author='Sam Green',
        author_email='cs51heads@gmail.com',
        license='MIT',
        packages=find_packages(),
        install_requires=[
            'pygments>=2.1.3',
            'pexpect>=4.2.1',
            ],
        entry_points={
            'console_scripts': [
                'caml-tex = camltex.command_line:main'
                ]
            },
        zip_safe=False,)

